<!DOCTYPE kpartgui>
<kpartgui version="1" name="kopete_nowlisteningchat">
	<MenuBar>
		<Menu name="tools">
			<text>&amp;Tools</text>
			<Action name="actionSendAdvert"/>
		</Menu>
	</MenuBar>
</kpartgui>
