<!DOCTYPE kpartgui>
<kpartgui name="Kdict" version="4">

<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Action name="file_save"/>
    <Action name="file_print"/>
    <Separator/>
    <Action name="start_query"/>
    <Action name="stop_query"/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
  <Menu noMerge="1" name="edit"><text>&amp;Edit</text>
    <Action name="edit_copy"/>
    <Action name="edit_select_all"/>
    <Separator/>
    <Action name="define_clipboard"/>
    <Action name="match_clipboard"/>
    <Separator/>
    <Action name="edit_find"/>
  </Menu>
  <Menu name="history"><text>Hist&amp;ory</text>
    <Action name="browse_back"/>
    <Action name="browse_forward"/>
    <Separator />
    <Action name="clear_history"/>
    <Separator />
    <ActionList name="history_items"/>
  </Menu>
  <Menu name="server"><text>Ser&amp;ver</text>
    <Action name="get_capabilities"/>
    <Action name="edit_sets"/>
    <Separator />
    <Menu name="database_sub"><text>Database &amp;Information</text>
       <Action name="db_summary"/>
       <Separator />
       <ActionList name="db_detail"/>
    </Menu>
    <Action name="strategy_info"/>
    <Action name="server_info"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="show_match" append="show_merge"/>
  </Menu>
</MenuBar>

<ToolBar fullWidth="true" newline="true" noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Action name="clear_query"/>
  <Action name="look_label"/>
  <Action name="query_combo"/>
  <Separator/>
  <Action name="define_btn"/>
  <Action name="match_btn"/>
</ToolBar>

<StatusBar/>

</kpartgui>
