/****************************************************************************
** Form interface generated from reading ui file './kopetepasswordwidgetbase.ui'
**
** Created: Sat Sep 9 15:27:20 2006
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.6   edited Aug 31 2005 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef KOPETEPASSWORDWIDGETBASE_H
#define KOPETEPASSWORDWIDGETBASE_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QCheckBox;
class QLabel;
class KPasswordEdit;

class KopetePasswordWidgetBase : public QWidget
{
    Q_OBJECT

public:
    KopetePasswordWidgetBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KopetePasswordWidgetBase();

    QCheckBox* mRemembered;
    QLabel* textLabel1;
    KPasswordEdit* mPassword;

protected:
    QGridLayout* KopetePasswordWidgetBaseLayout;
    QSpacerItem* spacer1;

protected slots:
    virtual void languageChange();

};

#endif // KOPETEPASSWORDWIDGETBASE_H
