<!DOCTYPE kpartgui>
<kpartgui version="30" name="kopetechatwindow">
	<MenuBar>
		<Menu noMerge="1" name="file">
			<text>&amp;Chat</text>
			<Action name="chat_send" />
			<Action name="file_save" />
			<Action name="file_print" />
			<Separator lineSeparator="true"/>
			<Action name="contacts_menu"/>
			<Merge/>
			<Separator lineSeparator="true"/>
			<Action name="tabs_close"/>
			<Action name="file_quit" />
		</Menu>

		<Menu name="format" >
			<text>&amp;Format</text>
			<Action name="format_smiley"/>
		</Menu>

		<Menu name="tabs" >
			<text>&amp;Tabs</text>
			<Action name="tabs_placement"/>
			<Action name="tabs_detach"/>
			<Action name="tabs_detachmove"/>
			<Action name="tabs_left"/>
			<Action name="tabs_right"/>
		</Menu>

		<Menu name="settings">
			<text>&amp;Settings</text>
			<Merge name="StandardToolBarMenuHandler" />
			<Menu append="show_merge" name="options_chatmembers">
				<text>&amp;Chat Members List</text>
				<Action name="options_membersleft" />
				<Action name="options_membersright" />
				<Action name="options_togglemembers" />
			</Menu>
			<Action name="enable_auto_spell_check" />
			<Action append="show_merge" name="options_styles"/>
			<Action append="configure_merge" name="settings_prefs" />
		</Menu>
		<Merge/>
	</MenuBar>

	<ToolBar name="mainToolBar" fullWidth="true">
		<text>Main Toolbar</text>
		<Action name="format_smiley"/>
		<Separator weakSeparator="true" lineSeparator="true"/>
		<Merge />
		<Separator weakSeparator="true" lineSeparator="true"/>
		<Action name="toolbar_animation"/>
	</ToolBar>

	<ToolBar name="statusToolBar">
		<text>Status</text>
	</ToolBar>
	<ToolBar name="formatToolBar">
		<text>Format Toolbar</text>
	</ToolBar>


</kpartgui>
