( cd opt/kde/share/doc/HTML/en/kpf ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kpf ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/krdc ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/krdc ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/krfb ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/krfb ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kwifimanager ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kwifimanager ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kopete ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kopete ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/lisa ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/lisa ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/knewsticker ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/knewsticker ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kget ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kget ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kppp ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kppp ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/ktalkd ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/ktalkd ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kcontrol/kcmtalkd ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kcontrol/kcmtalkd ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kcontrol/lanbrowser ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kcontrol/lanbrowser ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/ksirc ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/ksirc ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kdict ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kdict ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/lib64 ; rm -rf libkopete.so.1 )
( cd opt/kde/lib64 ; ln -sf libkopete.so.1.0.0 libkopete.so.1 )
( cd opt/kde/lib64 ; rm -rf libkopete.so )
( cd opt/kde/lib64 ; ln -sf libkopete.so.1.0.0 libkopete.so )
( cd opt/kde/lib64 ; rm -rf libkopete_videodevice.so.0 )
( cd opt/kde/lib64 ; ln -sf libkopete_videodevice.so.0.0.0 libkopete_videodevice.so.0 )
( cd opt/kde/lib64 ; rm -rf libkopete_videodevice.so )
( cd opt/kde/lib64 ; ln -sf libkopete_videodevice.so.0.0.0 libkopete_videodevice.so )
( cd opt/kde/lib64 ; rm -rf libkopete_msn_shared.so.0 )
( cd opt/kde/lib64 ; ln -sf libkopete_msn_shared.so.0.0.0 libkopete_msn_shared.so.0 )
( cd opt/kde/lib64 ; rm -rf libkopete_msn_shared.so )
( cd opt/kde/lib64 ; ln -sf libkopete_msn_shared.so.0.0.0 libkopete_msn_shared.so )
( cd opt/kde/lib64 ; rm -rf libkopete_oscar.so.2 )
( cd opt/kde/lib64 ; ln -sf libkopete_oscar.so.2.0.0 libkopete_oscar.so.2 )
( cd opt/kde/lib64 ; rm -rf libkopete_oscar.so )
( cd opt/kde/lib64 ; ln -sf libkopete_oscar.so.2.0.0 libkopete_oscar.so )
( cd opt/kde/lib64 ; rm -rf librss.so.1 )
( cd opt/kde/lib64 ; ln -sf librss.so.1.0.0 librss.so.1 )
( cd opt/kde/lib64 ; rm -rf librss.so )
( cd opt/kde/lib64 ; ln -sf librss.so.1.0.0 librss.so )
