/****************************************************************************
** Form interface generated from reading ui file './fileconfirmbase.ui'
**
** Created: Sat Sep 9 15:27:20 2006
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.6   edited Aug 31 2005 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef FILECONFIRMBASE_H
#define FILECONFIRMBASE_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QLabel;
class KLineEdit;
class KPushButton;
class QTextEdit;

class FileConfirmBase : public QWidget
{
    Q_OBJECT

public:
    FileConfirmBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~FileConfirmBase();

    QLabel* TextLabel1;
    QLabel* TextLabel1_2;
    QLabel* TextLabel7;
    KLineEdit* m_saveto;
    KPushButton* cmdBrowse;
    QLabel* TextLabel11;
    QLabel* TextLabel8;
    QTextEdit* m_description;
    QLabel* TextLabel13;
    KLineEdit* m_from;
    KLineEdit* m_filename;
    KLineEdit* m_size;

protected:
    QGridLayout* FileConfirmBaseLayout;
    QSpacerItem* Spacer6;

protected slots:
    virtual void languageChange();

};

#endif // FILECONFIRMBASE_H
