<!DOCTYPE kpartgui>
<kpartgui version="16" name="kopeteemailwindow">
	<MenuBar>
		<Menu noMerge="1" name="file">
			<text>&amp;Chat</text>
			<Action name="chat_send" />
			<Separator lineSeparator="true"/>
			<Merge />
			<Separator weakSeparator="true" lineSeparator="true"/>
			<Action name="file_quit" />
		</Menu>

		<Menu name="format" >
			<text>&amp;Format</text>
			<Action name="format_font" />
			<Action name="format_color" />
			<Action name="format_bgcolor" />
			<Action name="format_smiley"/>
		</Menu>

		<Menu name="settings">
			<text>&amp;Settings</text>
		</Menu>
		
		<Merge/>
	</MenuBar>

	<ToolBar name="mainToolBar" fullWidth="true" >
		<text>Main Toolbar</text>
		<Action name="format_font" />
		<Action name="format_color" />
		<Action name="format_bgcolor" />
		<Action name="format_smiley"/>
		<Separator weakSeparator="true" lineSeparator="true"/>
		<Merge/>
		<Separator weakSeparator="true" lineSeparator="true"/>
		<Action name="toolbar_animation"/>
	</ToolBar>

	<ToolBar name="statusToolBar">
		<text>Status</text>
	</ToolBar>
</kpartgui>
