<!DOCTYPE kpartgui>
<kpartgui name="kopete" version="22">
	<MenuBar>
		<Menu name="file" noMerge="1">
			<text>&amp;File</text>
			<!-- <Action name="Connection"/> -->
			<Action name="Status"/>
			<Separator lineSeparator="true"/>
			<Action name="AddContact"/>
			<Action name="AddGroup"/>
			<Action name="ExportContacts"/>
			<Separator lineSeparator="true"/>
			<Action name="file_quit"/>
		</Menu>
		<Menu name="edit">
			<text>&amp;Edit</text>
			<Action name="contactSendMessage" />
			<Action name="contactStartChat" />
			<Action name="contactSendFile" />
			<Separator lineSeparator="true" />
			<Action name="contactMove" />
			<Action name="contactCopy" />
			<Action name="contactRemove" />
			<Action name="contactRename" />
			<Action name="contactAddContact" />
			<Action name="contactAddTemporaryContact" />
			<Separator lineSeparator="true" />
			<Merge />
			<Separator lineSeparator="true" />
			<Action name="contactProperties" />

		</Menu>
		<Menu name="settings">
			<text>&amp;Settings</text>
			<Action name="settings_showmenubar" append="show_merge"/>
			<Merge name="StandardToolBarMenuHandler" append="show_merge"/>
			<Action name="settings_showstatusbar" append="show_merge"/>
                        <Action name="settings_show_offliners" append="show_merge"/>
                        <Action name="settings_show_empty_groups" append="show_merge"/>
                        <Action name="settings_plugins" append="configure_merge"/>
                        <Action name="settings_notifications" append="configure_merge"/>
                        <Action name="settings_global" append="configure_merge"/>
                        <Action name="settings_keys" append="configure_merge"/>
                        <Action name="settings_toolbars" append="configure_merge"/>
                        <Action name="settings_prefs" append="configure_merge"/>
		</Menu>
	</MenuBar>

	<ToolBar fullWidth="true" name="mainToolBar" noMerge="1"><Text>Main Toolbar</Text>
		<Action name="Status"/>
		<Action name="AddContact"/>
		<Action name="settings_show_offliners"/>
		<Action name="settings_show_empty_groups"/>
	</ToolBar>

	<ToolBar fullWidth="true" name="quickSearchBar" noMerge="1"><Text>Quick Search Bar</Text>
		<Action name="quicksearch_reset"/>
		<Action name="quicksearch_label"/>
		<Action name="quicksearch_bar"/>
	</ToolBar>
	
	<ToolBar fullWidth="true" name="editGlobalIdentityBar" noMerge="1" hidden="true" newline="true"><Text>Global Identity Bar</Text>
		<Action name="editglobal_widget"/>
	</ToolBar>

	<Menu name="contact_popup">
		<Action name="contactSendMessage" />
		<Action name="contactStartChat" />

		<Menu name="contact_popup_actions">
			  <text>&amp;Other Actions</text>
			  <Action name="contactSendFile" />
			  <Action name="contactSendEmail" />
			  <Action name="contactSyncKABC"/>
		</Menu>
		<Separator lineSeparator="true" />
		<Menu name="contact_popup_groups">
			  <text>&amp;Groups</text>
			  <Action name="contactMove" />
			  <Action name="contactCopy" />
		</Menu>
		<Action name="contactRemove" />
		<Action name="contactRename" />
		<Action name="contactAddContact" />
		<Action name="contactAddTemporaryContact" />
		<Separator lineSeparator="true" />
		<Merge />
		<Separator lineSeparator="true" />
		<Action name="contactProperties" />
		<Separator lineSeparator="true" />
	</Menu>
	<Menu name="group_popup">
		<Action name="contactRemove" />
		<Action name="contactRename" />
		<Action name="contactSendMessage" />
		<Action name="contactAddContact" />
		<Separator lineSeparator="true" />
		<Action name="contactProperties" />
	</Menu>
	<Menu name="contactlistitems_popup">
		<Action name="contactRemove" />
	</Menu>
	<Menu name="contactlist_popup">
		<Action name="AddContact"/>
		<Action name="AddGroup"/>
		<Separator lineSeparator="true"/>
		<Action name="settings_showmenubar" />
		<Action name="settings_show_offliners" />
		<Action name="settings_show_empty_groups" />
	</Menu>
</kpartgui>
