<!DOCTYPE kpartgui>
<kpartgui name="kopete_history" version="1">
	<MenuBar>
		<Menu name="edit">
			<text>&amp;Edit</text>
			<Action name="viewMetaContactStatistics" />
		</Menu>
	</MenuBar>
	<Menu name="contact_popup">
		<Action name="viewMetaContactStatistics" />
  	</Menu>
</kpartgui>
