<!DOCTYPE kpartgui>
<kpartgui version="5" name="kopete_msn_chat">
	<MenuBar>
		<Menu noMerge="1" name="file">
			<text>&amp;Chat</text>
			<Action name="msnInvite" />
			<Action name="msnRequestDisplayPicture" />
			<Action name="msnSendNudge" />
<!--			<Menu name="debug">
				<text>&amp;Debug</text>
				<Action name="msnDebugRawCommand" />
			</Menu> -->
			<Menu name="webcam">
				<text>&amp;Webcam</text>
				<Action name="msnWebcamSend" />
				<Action name="msnWebcamReceive" />
			</Menu> 
		</Menu>
	</MenuBar>


	<ToolBar name="statusToolBar">
		<Action name="msnDisplayPicture" />
	</ToolBar>


</kpartgui>
