<!DOCTYPE kpartgui>
<kpartgui name="kget" version="4">
<MenuBar>
    <Menu noMerge="1" name="file"><text>&amp;File</text>
        <Action name="open_transfer"/>
        <Action name="paste_transfer"/>
        <Separator/>
        <Action name="export_transfers"/>
        <Action name="import_transfers"/>
        <Action name="import_text"/>
        <Separator/>
        <Action name="quit"/>
    </Menu>
    <Menu name="transfer"><text>&amp;Transfer</text>
        <Action name="copy_url"/>
        <Action name="open_individual"/>
        <Separator/>
        <Action name="move_begin"/>
        <Action name="move_end"/>
        <Separator/>
        <Action name="resume"/>
        <Action name="pause"/>
        <Action name="delete"/>
        <Action name="restart"/>
        <Separator/>
        <Action name="queue"/>
        <Action name="timer"/>
        <Action name="delay"/>
    </Menu>
    <Menu noMerge="1" name="view"><text>&amp;View</text>
        <Action name="toggle_log"/>
    </Menu>
    <Menu name="options"><text>&amp;Options</text>
        <Action name="toggle_animation"/>
        <Action name="toggle_sound"/>
        <Separator/>
        <Action name="expert_mode"/>
        <Action name="use_last_dir"/>
        <Action name="offline_mode"/>
        <Separator/>
        <Action name="auto_disconnect"/>
        <Action name="auto_shutdown"/>
        <Action name="auto_paste"/>
    </Menu>
    <Menu name="settings"><text>&amp;Settings</text>
        <Action name="drop_target"/>
        <Action name="show_statusbar"/>
        <Action name="settings_showmenubar" append="show_merge"/>
    </Menu>
    <Menu name="help"><text>&amp;Help</text>
    </Menu>
</MenuBar>
<ToolBar fullWidth="true" name="mainToolBar"><text>Main Toolbar</text>
    <Action name="open_transfer" />
    <Action name="paste_transfer"/>
    <Separator/>
    <Action name="resume"/>
    <Action name="pause"/>
    <Action name="delete"/>
    <Action name="restart"/>
    <Separator/>
    <Action name="queue"/>
    <Action name="delay"/>
    <Separator/>
    <Action name="auto_disconnect"/>
    <Action name="auto_shutdown"/>
    <Action name="offline_mode"/>
    <Separator/>
    <Action name="drop_target"/>
</ToolBar>
</kpartgui>
