/****************************************************************************
** Form interface generated from reading ui file './kopeteawaydialogbase.ui'
**
** Created: Sat Sep 9 15:27:20 2006
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.6   edited Aug 31 2005 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef KOPETEAWAYDIALOG_BASE_H
#define KOPETEAWAYDIALOG_BASE_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QLabel;
class KLineEdit;
class KComboBox;

class KopeteAwayDialog_Base : public QWidget
{
    Q_OBJECT

public:
    KopeteAwayDialog_Base( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KopeteAwayDialog_Base();

    QLabel* TextLabel1;
    KLineEdit* txtOneShot;
    KComboBox* cmbHistory;

protected:
    QVBoxLayout* KopeteAwayDialog_BaseLayout;
    QSpacerItem* spacer2;

protected slots:
    virtual void languageChange();

};

#endif // KOPETEAWAYDIALOG_BASE_H
