/****************************************************************************
** Form interface generated from reading ui file './kopetepassworddialog.ui'
**
** Created: Sat Sep 9 15:27:20 2006
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.6   edited Aug 31 2005 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef KOPETEPASSWORDDIALOG_H
#define KOPETEPASSWORDDIALOG_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QLabel;
class KActiveLabel;
class KPasswordEdit;
class QCheckBox;

class KopetePasswordDialog : public QWidget
{
    Q_OBJECT

public:
    KopetePasswordDialog( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KopetePasswordDialog();

    QLabel* m_image;
    KActiveLabel* m_text;
    QLabel* textLabel3;
    KPasswordEdit* m_password;
    QCheckBox* m_save_passwd;

protected:
    QVBoxLayout* KopetePasswordDialogLayout;
    QSpacerItem* spacer1;
    QHBoxLayout* layout3;
    QHBoxLayout* layout2;

protected slots:
    virtual void languageChange();

};

#endif // KOPETEPASSWORDDIALOG_H
